{-# OPTIONS_GHC -fglasgow-exts -fallow-undecidable-instances -fallow-overlapping-instances #-}
module Hawl.Session.SessionT (SessionT(..), MonadSession(..),
                              runSessionT,
                              Session,
                              currentSessionID,currentSessionUser,
                              currentSessionStarted,currentSessionLastUse,
                              setCurrentSessionUser) where

import Hawl.Session.Types
import Hawl.Session.Operations
import Hawl.User.Types

import Database.HaskellDB (Database)

import Control.Monad.Reader
import Control.Monad.State
import System.Time (CalendarTime)

newtype SessionT m a = SessionT { unSessionT :: StateT Session m a }
    deriving (Functor, Monad, MonadTrans, MonadIO)

class Monad m => MonadSession m where
    currentSession :: m Session
    updateCurrentSession :: Session -> m ()

instance Monad m => MonadSession (SessionT m) where
    currentSession = SessionT get
    updateCurrentSession = SessionT . put

instance (MonadTrans t, MonadSession m, Monad (t m)) => MonadSession (t m) where
    currentSession = lift currentSession
    updateCurrentSession = lift . updateCurrentSession

instance MonadReader r m => MonadReader r (SessionT m) where
    ask = SessionT ask
    local f = SessionT . local f . unSessionT

runSessionT :: SessionT m a -> Session -> m (a, Session)
runSessionT = runStateT . unSessionT

currentSessionID :: MonadSession m => m SessionID
currentSessionID = liftM sessionID currentSession

currentSessionUser :: MonadSession m => m (Maybe UserID)
currentSessionUser = liftM sessionUser currentSession

currentSessionStarted :: MonadSession m => m CalendarTime
currentSessionStarted = liftM sessionStarted currentSession

currentSessionLastUse :: MonadSession m => m CalendarTime
currentSessionLastUse = liftM sessionLastUse currentSession

setCurrentSessionUser :: (MonadIO m, MonadSession m) => 
                         Database -> Maybe UserID -> m ()
setCurrentSessionUser db muid = 
    do s <- currentSession
       s' <- setSessionUser db s muid 
       updateCurrentSession s'
