module Hawl.Util.List where

import Data.Char
import Data.List

import Hawl.Util

-- Modified version of a function which is originally
-- Copyright Bryn Keller
splitBy :: (a -> Bool) -> [a] -> [[a]]
splitBy _ [] = []
splitBy f list = first : splitBy f (dropWhile f rest)
   where (first, rest) = break f list

-- | Replace all instances of a value in a list by another value.
replace :: Eq a =>
           a   -- ^ Value to look for
        -> a   -- ^ Value to replace it with
        -> [a] -- ^ Input list
        -> [a] -- ^ Output list
replace x y = map (\z -> if z == x then y else z)

glue :: [a] -> [[a]] -> [a]
glue g = concat . intersperse g

-- | Sort a list and remove all duplicates.
sortNub :: Ord a => [a] -> [a]
sortNub = map head . group . sort

-- | Sort a list and remove all duplicates, given an ordering.
sortNubBy :: (a -> a -> Ordering) -> [a] -> [a]
sortNubBy f = map head . sortGroupBy f

-- | Sorts and then groups elements given an ordering.
sortGroupBy :: (a -> a -> Ordering) -> [a] -> [[a]]
sortGroupBy f = groupBy (ordEq f) . sortBy f

-- | Group tuples by their first elements.
buildMultiMap :: Ord a => [(a,b)] -> [(a,[b])]
buildMultiMap =
    map (\g -> (fst (head g), map snd g) ) . sortGroupBy (compareBy fst)

trimBy :: (a -> Bool) -> [a] -> [a]
trimBy p = reverse . dropWhile p . reverse . dropWhile p

trimSpace :: String -> String
trimSpace = trimBy isSpace