{-# OPTIONS_GHC -fallow-overlapping-instances #-}
module Main where

import Network.NewCGI hiding (Html)
import Text.XHtml hiding (start)

import Control.Exception
import Control.Monad
import Control.Monad.Trans (MonadIO, liftIO)
import Data.FunctorM

import Hawl.User
import Hawl.Session
import Hawl.Util.CGI
import Hawl.Util.XHtml

import DBConnect

import Database.HaskellDB (Database)

--
-- * Session stuff
--

sessionUsername :: MonadIO m => Database -> Session -> m Username
sessionUsername db s = maybe (return "Anonymous") (getUsername db) (sessionUser s)

sessionCookieName :: String
sessionCookieName = "session"

loadSession :: Database -> Maybe UserID -> CGI (Maybe Session)
loadSession db muid = 
    do msi <- readCookie sessionCookieName
       msess <- maybe (return Nothing) (getSession db) msi
       fmapM (\s -> setSessionUser db s muid) msess

startSession :: Database -> Maybe UserID -> CGI Session
startSession db muid =
    do s <- newSession db muid
       setCookie $ newCookie sessionCookieName (show (sessionID s))
       return s

loadOrStartSession :: Database -> Maybe UserID -> CGI Session
loadOrStartSession db muid = 
    loadSession db muid >>= maybe (startSession db muid) return

--
-- * User / session actions
--

data UserAction = Login Username Password
                | Logout
                | Register Username Password
                | NoAction

getUserAction :: CGI UserAction
getUserAction =
    do
    ma <- getInput "user_action"
    case ma of
            Nothing -> return NoAction
            Just "login" -> liftM (uncurry Login) getCreds
            Just "logout" -> return Logout
            Just "register" -> liftM (uncurry Register) getCreds
            Just x -> fail $ "Unknown user_action: " ++ x
  where getCreds = do
                   [u,p] <- getManyInputs ["username","password"]
                   return (u,p)

start :: Database -> UserAction -> CGI Session
start db (Login u p) = 
    do muid <- authenticate db u p
       loadOrStartSession db muid
start db Logout = 
    do ms <- loadSession db Nothing
       maybe (return ()) (deleteSession db . sessionID) ms
       startSession db Nothing
start db (Register u p) =
    do muid <- addUser db u p
       case muid of
           Nothing -> fail $ "Couldn't register user " ++ u
           Just uid -> loadOrStartSession db (Just uid)
start db NoAction = loadOrStartSession db Nothing

main :: IO ()
main = runCGI cgiMain

cgiMain :: CGI CGIResult
cgiMain = handleExceptionCGI (dbConnect dbMain) cgiErrorHandler

dbMain :: Database -> CGI CGIResult
dbMain db =
    do
    a <- getUserAction
    sess <- start db a
    name <- sessionUsername db sess
    let page = mkPage sess name
    output $ renderHtml page


loginForm :: Html
loginForm = 
    form ! [method "post"]
             << [hidden "user_action" "login",
                 thediv << (thespan << "Username: " +++ textfield "username"),
                 thediv << (thespan << "Password: " +++ password "password"),
                 thediv << [submit "submit" "Login"]]

addUserForm :: Html
addUserForm = 
    form ! [method "post"]
             << [hidden "user_action" "register",
                 thediv << (thespan << "Username: " +++ textfield "username"),
                 thediv << (thespan << "Password: " +++ password "password"),
                 thediv << [submit "submit" "Register"]]

logoutLink :: Html
logoutLink = hlink "?user_action=logout" << "Log out"

mkPage :: Session -> Username -> Html
mkPage sess name = mkHtml hdr bdy
  where
  hdr :: [Html]
  hdr = [thetitle << "Session / User demo"]
  bdy = [p << ("Session ID: " ++ show (sessionID sess)),
         p << ("User ID:    " ++ maybe "N/A" show (sessionUser sess)),
         p << ("Username:   " ++ name),
         hr,
         h2 << "Log in",
         thediv << loginForm,
         hr,
         h2 << "Register",
         thediv << addUserForm,
         hr,
         h2 << "Log out",
         thediv << logoutLink]

